@@ -84,8 +84,7 @@ class Saml2Client(Base):
             nameid_format=None, scoping=None, consent=None, extensions=None,
             sign=None, response_binding=saml2.BINDING_HTTP_POST, **kwargs):
         """ Makes all necessary preparations for an authentication request
-        that negotiates
-        which binding to use for authentication.
+        that negotiates which binding to use for authentication.
 
         :param entityid: The entity ID of the IdP to send the request to
         :param relay_state: To where the user should be returned after
@@ -126,7 +125,7 @@ class Saml2Client(Base):
                 args = {}
 
             http_info = self.apply_binding(binding, _req_str, destination,
-                                           relay_state, **args)
+                                           relay_state, sign=sign, **args)
 
             return reqid, binding, http_info
         else:
@@ -241,7 +240,7 @@ class Saml2Client(Base):
                 relay_state = self._relay_state(req_id)
 
                 http_info = self.apply_binding(binding, srequest, destination,
-                                               relay_state, sigalg=sigalg)
+                                               relay_state, sign=sign, sigalg=sigalg)
 
                 if binding == BINDING_SOAP:
                     response = self.send(**http_info)
@@ -262,7 +261,7 @@ class Saml2Client(Base):
                                           "entity_ids": entity_ids,
                                           "name_id": code(name_id),
                                           "reason": reason,
-                                          "not_on_of_after": expire,
+                                          "not_on_or_after": expire,
                                           "sign": sign}
 
                     responses[entity_id] = (binding, http_info)
@@ -466,7 +465,7 @@ class Saml2Client(Base):
                                   attribute=attribute,
                                   sp_name_qualifier=sp_name_qualifier,
                                   name_qualifier=name_qualifier,
-                                  nameid_format=nameid_format,
+                                  format=nameid_format,
                                   response_args=response_args)
         elif binding == BINDING_HTTP_POST:
             mid = sid()
@@ -479,7 +478,7 @@ class Saml2Client(Base):
                                     "sign": sign}
             relay_state = self._relay_state(query.id)
             return self.apply_binding(binding, "%s" % query, destination,
-                                      relay_state)
+                                      relay_state, sign=sign)
         else:
             raise SAMLError("Unsupported binding")
 
@@ -524,7 +523,7 @@ class Saml2Client(Base):
 
         if binding == BINDING_SOAP:
             response_bindings = [BINDING_SOAP]
-        elif binding == BINDING_HTTP_POST or BINDING_HTTP_REDIRECT:
+        elif binding in [BINDING_HTTP_POST, BINDING_HTTP_REDIRECT]:
             response_bindings = [BINDING_HTTP_POST, BINDING_HTTP_REDIRECT]
         else:
             response_bindings = self.config.preferred_binding[
@@ -536,4 +535,4 @@ class Saml2Client(Base):
 
         return self.apply_binding(rinfo["binding"], response,
                                   rinfo["destination"], relay_state,
-                                  response=True)
+                                  response=True, sign=sign)
