@@ -69,6 +69,7 @@ COMMON_ARGS = [
     "allow_unknown_attributes",
     "crypto_backend",
     "id_attr_name",
+    "delete_tmpfiles",
 ]
 
 SP_ARGS = [
@@ -243,6 +244,7 @@ class Config(object):
         self.attribute = []
         self.attribute_profile = []
         self.requested_attribute_name_format = NAME_FORMAT_URI
+        self.delete_tmpfiles = True
 
     def setattr(self, context, attr, val):
         if context == "":
@@ -358,6 +360,12 @@ class Config(object):
             except TypeError:  # Something that can't be a string
                 setattr(self, arg, cnf[arg])
 
+        if not self.delete_tmpfiles:
+            logger.warning(
+                "delete_tmpfiles is set to False; "
+                "temporary files will not be deleted."
+            )
+
         if "service" in cnf:
             for typ in ["aa", "idp", "sp", "pdp", "aq"]:
                 try:
