@@ -33,20 +33,53 @@ from saml2.s_utils import UnknownSystemEntity
 from saml2.sigver import split_len
 from saml2.validate import valid_instance
 from saml2.time_util import valid
+from saml2.time_util import instant
+from saml2.time_util import add_duration
+from saml2.time_util import before
+from saml2.time_util import str_to_time
 from saml2.validate import NotValid
 from saml2.sigver import security_context
+from saml2.extension.mdattr import NAMESPACE as NS_MDATTR
+from saml2.extension.mdattr import EntityAttributes
+from saml2.extension.algsupport import NAMESPACE as NS_ALGSUPPORT
+from saml2.extension.algsupport import SigningMethod, DigestMethod
+from saml2.extension.mdui import NAMESPACE as NS_MDUI
+from saml2.extension.mdui import UIInfo
+from saml2.extension.mdui import DisplayName
+from saml2.extension.mdui import Description
+from saml2.extension.mdui import InformationURL
+from saml2.extension.mdui import PrivacyStatementURL
+from saml2.extension.mdui import Logo
 
-__author__ = 'rolandh'
 
 logger = logging.getLogger(__name__)
 
+classnames = {
+    "mdattr_entityattributes": "{ns}&{tag}".format(
+        ns=NS_MDATTR, tag=EntityAttributes.c_tag
+    ),
+    "algsupport_signing_method": "{ns}&{tag}".format(ns=NS_ALGSUPPORT, tag=SigningMethod.c_tag),
+    "algsupport_digest_method": "{ns}&{tag}".format(ns=NS_ALGSUPPORT, tag=DigestMethod.c_tag),
+    "mdui_uiinfo": "{ns}&{tag}".format(ns=NS_MDUI, tag=UIInfo.c_tag),
+    "mdui_uiinfo_display_name": "{ns}&{tag}".format(ns=NS_MDUI, tag=DisplayName.c_tag),
+    "mdui_uiinfo_description": "{ns}&{tag}".format(ns=NS_MDUI, tag=Description.c_tag),
+    "mdui_uiinfo_information_url": "{ns}&{tag}".format(
+        ns=NS_MDUI, tag=InformationURL.c_tag
+    ),
+    "mdui_uiinfo_privacy_statement_url": "{ns}&{tag}".format(
+        ns=NS_MDUI, tag=PrivacyStatementURL.c_tag
+    ),
+    "mdui_uiinfo_logo": "{ns}&{tag}".format(ns=NS_MDUI, tag=Logo.c_tag),
+}
 
-class ToOld(Exception):
-    pass
+ENTITY_CATEGORY = "http://macedir.org/entity-category"
+ENTITY_CATEGORY_SUPPORT = "http://macedir.org/entity-category-support"
+ASSURANCE_CERTIFICATION = "urn:oasis:names:tc:SAML:attribute:assurance-certification"
+
+SAML_METADATA_CONTENT_TYPE = "application/samlmetadata+xml"
+DEFAULT_FRESHNESS_PERIOD = "P0Y0M0DT12H0M0S"
 
 
-class SourceNotFound(Exception):
-    pass
 
 REQ2SRV = {
     # IDP
@@ -70,12 +103,18 @@ REQ2SRV = {
     "discovery_service_request": "discovery_response"
 }
 
-ENTITYATTRIBUTES = "urn:oasis:names:tc:SAML:metadata:attribute&EntityAttributes"
-ENTITY_CATEGORY = "http://macedir.org/entity-category"
-ENTITY_CATEGORY_SUPPORT = "http://macedir.org/entity-category-support"
+
+class ToOld(Exception):
+    pass
 
 
-# ---------------------------------------------------
+class TooOld(ToOld):
+    pass
+
+
+class SourceNotFound(Exception):
+    pass
+
 
 def load_extensions():
     from saml2 import extension
@@ -359,7 +398,7 @@ class MetaData(object):
         res = []
         if "extensions" in self[entity_id]:
             for elem in self[entity_id]["extensions"]["extension_elements"]:
-                if elem["__class__"] == ENTITYATTRIBUTES:
+                if elem["__class__"] == classnames["mdattr_entityattributes"]:
                     for attr in elem["attribute"]:
                         res.append(attr["text"])
 
@@ -471,7 +510,7 @@ class InMemoryMetaData(MetaData):
         if self.check_validity:
             try:
                 if not valid(entity_descr.valid_until):
-                    logger.error("Entity descriptor (entity id:%s) to old",
+                    logger.error("Entity descriptor (entity id:%s) too old",
                                  entity_descr.entity_id)
                     self.to_old.append(entity_descr.entity_id)
                     return
@@ -535,7 +574,7 @@ class InMemoryMetaData(MetaData):
             if self.check_validity:
                 try:
                     if not valid(self.entities_descr.valid_until):
-                        raise ToOld(
+                        raise TooOld(
                             "Metadata not valid anymore, it's only valid "
                             "until %s" % (
                                 self.entities_descr.valid_until,))
@@ -634,22 +673,21 @@ class InMemoryMetaData(MetaData):
     def parse_and_check_signature(self, txt):
         self.parse(txt)
 
-        if self.cert:
-            if not self.signed():
-                return True
-
-            node_name = self.node_name \
-                        or "%s:%s" % (md.EntitiesDescriptor.c_namespace,
-                                      md.EntitiesDescriptor.c_tag)
-
-            if self.security.verify_signature(
-                    txt, node_name=node_name, cert_file=self.cert):
-                return True
-            else:
-                return False
-        else:
+        if not self.cert:
             return True
 
+        if not self.signed():
+            return True
+
+        fallback_name = "{ns}:{tag}".format(
+            ns=md.EntitiesDescriptor.c_namespace, tag=md.EntitiesDescriptor.c_tag
+        )
+        node_name = self.node_name or fallback_name
+
+        return self.security.verify_signature(
+            txt, node_name=node_name, cert_file=self.cert
+        )
+
 
 class MetaDataFile(InMemoryMetaData):
     """
@@ -775,9 +813,6 @@ class MetaDataMD(InMemoryMetaData):
             self.entity[key] = item
 
 
-SAML_METADATA_CONTENT_TYPE = 'application/samlmetadata+xml'
-
-
 class MetaDataMDX(InMemoryMetaData):
     """
     Uses the MDQ protocol to fetch entity information.
@@ -787,11 +822,12 @@ class MetaDataMDX(InMemoryMetaData):
 
     @staticmethod
     def sha1_entity_transform(entity_id):
-        return "{{sha1}}{}".format(
-            hashlib.sha1(entity_id.encode("utf-8")).hexdigest())
+        entity_id_sha1 = hashlib.sha1(entity_id.encode("utf-8")).hexdigest()
+        transform = "{{sha1}}{digest}".format(digest=entity_id_sha1)
+        return transform
 
     def __init__(self, url=None, security=None, cert=None,
-                 entity_transform=None, **kwargs):
+                 entity_transform=None, freshness_period=None, **kwargs):
         """
         :params url: mdx service url
         :params security: SecurityContext()
@@ -801,6 +837,8 @@ class MetaDataMDX(InMemoryMetaData):
         hash) the entity id. It is applied to the entity id before it is
         concatenated with the request URL sent to the MDX server. Defaults to
         sha1 transformation.
+        :params freshness_period: a duration in the format described at
+        https://www.w3.org/TR/xmlschema-2/#duration
         """
         super(MetaDataMDX, self).__init__(None, **kwargs)
         if not url:
@@ -815,6 +853,8 @@ class MetaDataMDX(InMemoryMetaData):
 
         self.cert = cert
         self.security = security
+        self.freshness_period = freshness_period or DEFAULT_FRESHNESS_PERIOD
+        self.expiration_date = {}
 
         # We assume that the MDQ server will return a single entity
         # described by a single <EntityDescriptor> element. The protocol
@@ -822,28 +862,53 @@ class MetaDataMDX(InMemoryMetaData):
         # <EntitiesDescriptor> element but we will not currently support
         # that use case since it is unlikely to be leveraged for most
         # flows.
-        self.node_name = "%s:%s" % (md.EntityDescriptor.c_namespace,
-                                      md.EntityDescriptor.c_tag)
+        self.node_name = "{ns}:{tag}".format(
+            ns=md.EntityDescriptor.c_namespace, tag=md.EntityDescriptor.c_tag
+        )
 
     def load(self, *args, **kwargs):
         # Do nothing
         pass
 
-    def __getitem__(self, item):
-        try:
-            return self.entity[item]
-        except KeyError:
-            mdx_url = "%s/entities/%s" % (self.url, self.entity_transform(item))
-            response = requests.get(mdx_url, headers={
-                'Accept': SAML_METADATA_CONTENT_TYPE})
-            if response.status_code == 200:
-                _txt = response.content
+    def _fetch_metadata(self, item):
+        mdx_url = "{url}/entities/{id}".format(
+            url=self.url, id=self.entity_transform(item)
+        )
 
-                if self.parse_and_check_signature(_txt):
-                    return self.entity[item]
-            else:
-                logger.info("Response status: %s", response.status_code)
-            raise KeyError
+        response = requests.get(mdx_url, headers={"Accept": SAML_METADATA_CONTENT_TYPE})
+        if response.status_code != 200:
+            error_msg = "Fething {item}: Got response status {status}".format(
+                item=item, status=response.status_code
+            )
+            logger.info(error_msg)
+            raise KeyError(error_msg)
+
+        _txt = response.content
+        if not self.parse_and_check_signature(_txt):
+            error_msg = "Fething {item}: invalid signature".format(
+                item=item, status=response.status_code
+            )
+            logger.info(error_msg)
+            raise KeyError(error_msg)
+
+        curr_time = str_to_time(instant())
+        self.expiration_date[item] = add_duration(curr_time, self.freshness_period)
+        return self.entity[item]
+
+    def _is_metadata_fresh(self, item):
+        return before(self.expiration_date[item])
+
+    def __getitem__(self, item):
+        if item not in self.entity:
+            entity = self._fetch_metadata(item)
+        elif not self._is_metadata_fresh(item):
+            msg = "Metadata for {} have expired; refreshing metadata".format(item)
+            logger.info(msg)
+            old_entity = self.entity.pop(item)
+            entity = self._fetch_metadata(item)
+        else:
+            entity = self.entity[item]
+        return entity
 
     def single_sign_on_service(self, entity_id, binding=None, typ="idpsso"):
         if binding is None:
@@ -930,9 +995,11 @@ class MetadataStore(MetaData):
                 key = kwargs['url']
                 url = kwargs['url']
                 cert = kwargs.get('cert')
+                freshness_period = kwargs.get('freshness_period', None)
                 security = self.security
                 entity_transform = kwargs.get('entity_transform', None)
-                _md = MetaDataMDX(url, security, cert, entity_transform)
+                _md = MetaDataMDX(url, security, cert, entity_transform,
+                                  freshness_period=freshness_period)
             else:
                 key = args[1]
                 url = args[1]
@@ -1222,6 +1289,15 @@ class MetadataStore(MetaData):
         attributes = self.entity_attributes(entity_id)
         return attributes.get(ENTITY_CATEGORY_SUPPORT, [])
 
+    def assurance_certifications(self, entity_id):
+        assurance_certifications = (
+            certification
+            for name, values in self.entity_attributes(entity_id).items()
+            if name == ASSURANCE_CERTIFICATION
+            for certification in values
+        )
+        return assurance_certifications
+
     def entity_attributes(self, entity_id):
         """
         Get all entity attributes for an entry in the metadata.
@@ -1243,7 +1319,7 @@ class MetadataStore(MetaData):
         except KeyError:
             return res
         for elem in ext["extension_elements"]:
-            if elem["__class__"] == ENTITYATTRIBUTES:
+            if elem["__class__"] == classnames["mdattr_entityattributes"]:
                 for attr in elem["attribute"]:
                     if attr["name"] not in res:
                         res[attr["name"]] = []
@@ -1251,6 +1327,167 @@ class MetadataStore(MetaData):
                         "attribute_value"]]
         return res
 
+    def supported_algorithms(self, entity_id):
+        """
+        Get all supported algorithms for an entry in the metadata.
+
+        Example return data:
+
+        {'digest_methods': ['http://www.w3.org/2001/04/xmldsig-more#sha224', 'http://www.w3.org/2001/04/xmlenc#sha256'],
+         'signing_methods': ['http://www.w3.org/2001/04/xmldsig-more#rsa-sha256']}
+
+        :param entity_id: Entity id
+        :return: dict with keys and value-lists from metadata
+
+        :type entity_id: string
+        :rtype: dict
+        """
+        res = {
+            'digest_methods': [],
+            'signing_methods': []
+        }
+        try:
+            ext = self.__getitem__(entity_id)["extensions"]
+        except KeyError:
+            return res
+        for elem in ext["extension_elements"]:
+            if elem["__class__"] == classnames["algsupport_digest_method"]:
+                res['digest_methods'].append(elem['algorithm'])
+            elif elem["__class__"] == classnames["algsupport_signing_method"]:
+                res['signing_methods'].append(elem['algorithm'])
+        return res
+
+    def _lookup_elements_by_cls(self, root, cls):
+        elements = (
+            element
+            for uiinfo in root
+            for element_key, elements in uiinfo.items()
+            if element_key != "__class__"
+            for element in elements
+            if element.get("__class__") == cls
+        )
+        return elements
+
+    def _lookup_elements_by_key(self, root, key):
+        elements = (
+            element
+            for uiinfo in root
+            for elements in [uiinfo.get(key, [])]
+            for element in elements
+        )
+        return elements
+
+    def mdui_uiinfo(self, entity_id):
+        try:
+            data = self[entity_id]
+        except KeyError:
+            data = {}
+
+        descriptor_names = (
+            item
+            for item in data.keys()
+            if item.endswith("_descriptor")
+        )
+        descriptors = (
+            descriptor
+            for descriptor_name in descriptor_names
+            for descriptor in self[entity_id].get(descriptor_name, [])
+        )
+        extensions = (
+            extension
+            for descriptor in descriptors
+            for extension in descriptor.get("extensions", {}).get("extension_elements", [])
+        )
+        uiinfos = (
+            extension
+            for extension in extensions
+            if extension.get("__class__") == classnames["mdui_uiinfo"]
+        )
+        return uiinfos
+
+    def _mdui_uiinfo_i18n_elements_lookup(self, entity_id, langpref, element_hint, lookup):
+        uiinfos = self.mdui_uiinfo(entity_id)
+        elements = lookup(uiinfos, element_hint)
+        lang_elements = (
+            element
+            for element in elements
+            if langpref is None or element.get("lang") == langpref
+        )
+        values = (
+            value
+            for element in lang_elements
+            for value in [element.get("text")]
+        )
+        return values
+
+    def mdui_uiinfo_i18n_element_cls(self, entity_id, langpref, element_cls):
+        values = self._mdui_uiinfo_i18n_elements_lookup(
+            entity_id, langpref, element_cls, self._lookup_elements_by_cls
+        )
+        return values
+
+    def mdui_uiinfo_i18n_element_key(self, entity_id, langpref, element_key):
+        values = self._mdui_uiinfo_i18n_elements_lookup(
+            entity_id, langpref, element_key, self._lookup_elements_by_key
+        )
+        return values
+
+    def mdui_uiinfo_display_name(self, entity_id, langpref=None):
+        cls = classnames["mdui_uiinfo_display_name"]
+        values = self.mdui_uiinfo_i18n_element_cls(entity_id, langpref, cls)
+        return values
+
+    def mdui_uiinfo_description(self, entity_id, langpref=None):
+        cls = classnames["mdui_uiinfo_description"]
+        values = self.mdui_uiinfo_i18n_element_cls(entity_id, langpref, cls)
+        return values
+
+    def mdui_uiinfo_information_url(self, entity_id, langpref=None):
+        cls = classnames["mdui_uiinfo_information_url"]
+        values = self.mdui_uiinfo_i18n_element_cls(entity_id, langpref, cls)
+        return values
+
+    def mdui_uiinfo_privacy_statement_url(self, entity_id, langpref=None):
+        cls = classnames["mdui_uiinfo_privacy_statement_url"]
+        values = self.mdui_uiinfo_i18n_element_cls(entity_id, langpref, cls)
+        return values
+
+    def mdui_uiinfo_logo(self, entity_id, width=None, height=None):
+        uiinfos = self.mdui_uiinfo(entity_id)
+        cls = classnames["mdui_uiinfo_logo"]
+        elements = self._lookup_elements_by_cls(uiinfos, cls)
+        values = (
+            element
+            for element in elements
+            if width is None or element.get("width") == width
+            if height is None or element.get("height") == height
+        )
+        return values
+
+    def contact_person_data(self, entity_id, contact_type=None):
+        try:
+            data = self[entity_id]
+        except KeyError:
+            data = {}
+
+        contacts = (
+            {
+                "contact_type": _contact_type,
+                "given_name": contact.get("given_name", {}).get("text", ""),
+                "email_address": [
+                    address
+                    for email in contact.get("email_address", {})
+                    for address in [email.get("text")]
+                    if address
+                ],
+            }
+            for contact in data.get("contact_person", [])
+            for _contact_type in [contact.get("contact_type", "")]
+            if contact_type is None or contact_type == _contact_type
+        )
+
+        return contacts
+
     def bindings(self, entity_id, typ, service):
         for _md in self.metadata.values():
             if entity_id in _md.items():
